package net.deterlab.abac;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;


/**
 * @author <a href="http://abac.deterlab.net">ISI ABAC team</a>
 * @version 1.3
 */
public class CredentialFactory {
    static List<Constructor<? extends Credential> > defCons = 
	new ArrayList<Constructor<? extends Credential> >();
    static List<Constructor<? extends Credential> > cons = 
	new ArrayList<Constructor<? extends Credential> >();
    static String defName = "net.deterlab.abac.GENICredential";
    public static final int maxSize = 50 * 1024;

    static void initDefaultConstructors() throws ABACException {
	try {
	    Class c = Class.forName(defName);
	    Constructor<?  extends Credential> con = c.getConstructor(
		    Class.forName("java.io.InputStream"),
		    Class.forName("java.util.Collection"));
	    defCons.add(con);
	}
	catch (Exception e) {
	    throw new ABACException("Could not initialize default " +
		    "credential parser.  Something is very wrong", e);
	}
    }

    static Credential[] newInstance(InputStream is, Set<Identity> ids) 
	    throws ABACException {
	Credential rv = null;
	ABACException err = null;
	List<Constructor<? extends Credential> > useCons = 
	    (cons.size()> 0 ) ? cons : defCons;

	if ( defCons.size() < 1 )  initDefaultConstructors();

	if (!is.markSupported() && useCons.size() > 1) 
	    is = new BufferedInputStream(is, maxSize);
	is.mark(maxSize);

	for (Constructor<? extends Credential> c : useCons ) {
	    try {
		rv = c.newInstance(is, ids);
	    }
	    catch (InvocationTargetException ie ) {
		Throwable th = ie.getCause();

		if ( th != null && th instanceof ABACException ) 
		    err = (ABACException) th;
		else
		    err = new ABACException("Constructor error " + 
				"(this is weird) " + ie.getMessage(), ie);
		rv = null;
	    }
	    catch (Exception e ) {
		err = new ABACException("Constructor error " + 
			e.getMessage(), e);
		rv = null;
	    }
	    if ( rv != null ) 
		break;

	    try {
		if (useCons.size() > 1) is.reset(); 
	    }
	    catch (IOException ie) { 
		break;
	    }
	}

	if ( rv != null ) 
	    return new Credential[] { rv };
	else 
	    throw (err != null) ? err :
		new ABACException("null exception and failed construction??");
    }


    static Credential[] newInstance(File f, Set<Identity> ids) 
	throws ABACException {
	try {
	    return newInstance(new FileInputStream(f), ids);
	}
	catch (FileNotFoundException e) {
	    throw new CertInvalidException(e.getMessage(), e);
	}
    }
    static Credential[] newInstance(String fn, Set<Identity> ids) 
	throws ABACException {
	return newInstance(new File(fn), ids);
    }

    static public void registerClass(String name)
	throws ClassNotFoundException, NoSuchMethodException, 
			  SecurityException {

	Class c = Class.forName(name);
	Constructor<?  extends Credential> con = c.getConstructor(
		Class.forName("java.io.InputStream"),
		Class.forName("java.util.Collection"));

	cons.add(con);
    }
}
